#!/bin/sh

#Set scanner
SCANNER=cbctl

# Load common utilities
. ./scan_utilities.sh

# e.g. pull_and_scan artifactory-pub.bit9.local:5000/re/releng/photon3:latest
pull_and_scan() {
	# Take the first argument as the IMAGE
	IMAGE_ARG=$1

	set_image_vars "${IMAGE_ARG}"

	# Remove "docker.io/" prefix for DockerHub images since that causes
	# problems with `docker images -q` later. See PSEC-2338
	IMAGE_ARG=$(echo "${IMAGE_ARG}" | sed "s:^docker.io/::")

	# Logs into ECR if the image is hosted there
	if echo "${IMAGE}" | grep -qi ".dkr.ecr."; then
		login_to_ecr "${IMAGE}"
	fi

	if ! docker pull "${IMAGE_ARG}"; then
		error_msg "Failed pulling ${IMAGE_ARG}"
		exit 2
	fi

	retry=0
	until [ "$retry" -ge 3 ]; do
		~/.cbctl/cbctl \
			image \
			scan \
			--force \
			"${IMAGE_ARG}" \
			--debug="${RES_CLI}.log" \
			-o json >"${RES_CLI}.json" &&
			break
		retry=$((retry + 1))
		sleep 1
	done

	if test ${retry} -ge 3; then
		error_msg "Failed scanning ${IMAGE_ARG} with cbctl"
		exit 3
	fi
}

# Enable debug output
set -x

# Display binary version
cbctl version

# Pull base images in advance
for IMAGE in ${BASE_IMAGES}; do
	if ! docker pull "${IMAGE}" 2>/dev/null; then
		error_msg "Failed pulling base image. Skipping: ${IMAGE}"
	fi
done

SCAN_STATUS=0
for IMAGE in ${SCAN_IMAGE_LIST}; do
	cleanup

	pull_and_scan "${IMAGE}"
	docker image inspect "${IMAGE}" >"${REP_PATH_CLI}/target_image.json"

	# Find base image, return first from BASE_IMAGES otherwise
	BASE=$(echo "${BASE_IMAGES}" | cut -d" " -f1)

	BASE_IMAGE_CHECK=true
	# Check if we are scanning an approved base image
	if check_is_re_image "${IMAGE}"; then
		# IMAGE is a base image. Do not diff against base
		BASE_IMAGE_CHECK=false
		BASE=""
	fi

	if ${BASE_IMAGE_CHECK} && test -n "${BASE_IMAGES}"; then
		for BI in ${BASE_IMAGES}; do
			if check_image_base "${IMAGE}" "${BI}"; then
				BASE="${BI}"
			fi
		done

		export BASE

		# Get the base image for diff-ing
		pull_and_scan "${BASE}"
		docker image inspect "${BASE}" >"${REP_PATH_CLI}/base_image.json"

		set_image_vars "${BASE}"
		BASE_SCAN_JSON="${RES_CLI}.json"
	fi
	export IMAGE

	set_image_vars "${IMAGE}"
	TARGET_SCAN_JSON="${RES_CLI}.json"

	INFO_OS=""
	INFO_OS=$(get_info_os "${IMAGE}")

	python3 analyser/scanResultApi.py \
		-t "${TARGET_SCAN_JSON}" \
		-b "${BASE_SCAN_JSON}" \
		-jb "${REP_PATH_CLI}/base_image.json" \
		-jt "${REP_PATH_CLI}/target_image.json" \
		-io "${INFO_OS}" \
		-oh "${RES_CLI}.html" \
		-oj "${JSON_OUTPUT}"
	LAST_STATUS=$?

	# If this scan or a previous scan failed, keep the status
	if [ ${SCAN_STATUS} -ne 0 ] || [ ${LAST_STATUS} -ne 0 ]; then
		SCAN_STATUS=1
	fi
done

exit ${SCAN_STATUS}
